@@ -1,42 +1,7 @@
+from re import compile, IGNORECASE
 from simpler.terminal import cprint
 from typing import Any, Generator, List, Optional, Union
 
-def _mysql_converter():
-	''' Simpler MySQL converter that returns some bytes as strings and decimals as floats. '''
-	from mysql.connector.constants import FieldFlag
-	from mysql.connector.conversion import MySQLConverter as NativeConverter
-
-	class MySQLConverter(NativeConverter):
-		def row_to_python(self, row, fields):
-			res = super().row_to_python(row, fields)
-			return res
-
-		def _BLOB_to_python(self, value, dsc=None):
-			"""Convert BLOB data type to Python"""
-			return self._STRING_to_python(value, dsc)
-		_LONG_BLOB_to_python = _BLOB_to_python
-		_MEDIUM_BLOB_to_python = _BLOB_to_python
-		_TINY_BLOB_to_python = _BLOB_to_python
-
-		def _DECIMAL_to_python(self, value, desc=None):
-			return float(value.decode(self.charset))
-		_NEWDECIMAL_to_python = _DECIMAL_to_python
-
-		def _float64_to_mysql(self, value, desc=None):
-			return float(value)
-
-		def _STRING_to_python(self, value, dsc=None):
-			res = super(MySQLConverter, self)._STRING_to_python(value, dsc)
-			if dsc[7] & FieldFlag.BINARY:
-				return res.decode(self.charset)
-			return res
-		_VAR_STRING_to_python = _STRING_to_python
-
-		def _timestamp_to_mysql(self, value, desc=None):
-			return value.strftime('%Y-%m-%d %H:%M:%S').encode(self.charset)
-
-	return MySQLConverter
-
 class SQL:
 	''' Connector for SQL databases with a handful of helpers. '''
 	ENGINES = 'mysql', 'mariadb', 'mssql', 'postgre'
@@ -100,10 +65,22 @@ class SQL:
 			self._connection['database'] = db
 
 	def _init_postgre(self, host, password, db):
+		from psycopg import ClientCursor
+		from json import dumps
+		from psycopg import adapters
+		from psycopg._oids import NUMERIC_OID
+		from psycopg.adapt import Dumper
+		from psycopg.types.numeric import FloatLoader
+		class DictDumper(Dumper):
+			def dump(self, value):
+				return dumps(value, ensure_ascii=False, separators=(',', ':')).encode('utf-8')
+		adapters.register_dumper(dict, DictDumper)
+		adapters.register_loader(NUMERIC_OID, FloatLoader)
 		self._connection.update({
-			'database': db,
+			'dbname': db,
 			'host': host,
 			'password': password,
+			'cursor_factory': ClientCursor
 		})
 
 	def close(self) -> None:
@@ -120,10 +97,10 @@ class SQL:
 			if self.engine == 'mysql':
 				try:
 					from mysql.connector import connect
+					if self.native_types:
+						self._connection['converter_class'] = _mysql_converter()
 				except ModuleNotFoundError:
 					raise ModuleNotFoundError('Missing MySQL/MariaDB connector. Install a mysql client and then do `pip install mysql.connector`.')
-				if self.native_types:
-					self._connection['converter_class'] = _mysql_converter()
 			elif self.engine == 'mariadb':
 				try:
 					from mariadb import connect
@@ -136,17 +113,9 @@ class SQL:
 					raise ModuleNotFoundError('Missing MS-SQL connector. Install a MS-SQL client and then do `pip install pymssql`.')
 			elif self.engine == 'postgre':
 				try:
-					from psycopg2 import connect
-					from psycopg2.extras import Json
-					from psycopg2.extensions import register_adapter, register_type, new_type, DECIMAL
-					register_adapter(dict, Json)
-					register_type(new_type(
-						DECIMAL.values,
-						'DEC2FLOAT',
-						lambda value, _: float(value) if value is not None else None
-					))
+					from psycopg import connect
 				except ModuleNotFoundError:
-					raise ModuleNotFoundError('Missing PostgreSQL connector. Install a PostgreSQL client and then do `pip install psycopg2-binary`.')
+					raise ModuleNotFoundError('Missing PostgreSQL connector. Install a PostgreSQL client and then do `pip install "psycopg[binary]"`.')
 			self._connection = connect(**self._connection)
 			self._cursor = self._connection.cursor(**self._cursor)
 			self._initialized = True
@@ -288,21 +257,47 @@ class SQL:
 		while self.max_insertions is not None and self.max_insertions < len(rows):
 			part, rows = rows[:self.max_insertions], rows[self.max_insertions:]
 			self.insert_all(table, part, tuple_rows, commit=False)
+		is_postgre = self.engine == 'postgre'
 		if tuple_rows:
-			query = 'INSERT INTO %s VALUES %s' % (
-				table,
-				','.join(['(' + ','.join(['%s'] * len(rows[0])) + ')'] * len(rows))
-			)
-			params = [param for row in rows for param in row]
+			if is_postgre:
+				query = 'INSERT INTO %s VALUES (%s) RETURNING *' % (
+					table,
+					','.join(['%s'] * len(rows[0]))
+				)
+			else:
+				query = 'INSERT INTO %s VALUES %s' % (
+					table,
+					','.join(['(' + ','.join(['%s'] * len(rows[0])) + ')'] * len(rows))
+				)
+			params = [param for row in rows for param in row] if not is_postgre else rows
 		else:
 			keys = list(rows[0].keys())
-			values = '(%s)' % ','.join('%s' for _ in keys)
-			query = 'INSERT INTO %s(%s) VALUES %s' % (
-				table,
-				','.join(keys),
-				','.join(values for _ in rows)
-			)
-			params = [insertion[key] for insertion in rows for key in keys]
+			if is_postgre:
+				query = 'INSERT INTO %s(%s) VALUES (%s) RETURNING *' % (
+					table,
+					','.join([str(key) for key in keys]),
+					','.join(['%s'] * len(keys))
+				)
+				params = [[row[key] for key in keys] for row in rows]
+			else:
+				values = '(%s)' % ','.join('%s' for _ in keys)
+				query = 'INSERT INTO %s(%s) VALUES %s' % (
+					table,
+					','.join(keys),
+					','.join(values for _ in rows)
+				)
+				params = [insertion[key] for insertion in rows for key in keys]
+		if is_postgre:
+			self.cursor().executemany(query, params, returning=True)
+			try:
+				ids = []
+				while True:
+					ids.append(self.cursor().fetchone()[0])
+					if not self.cursor().nextset():
+						break
+				return ids[-1] if ids else None
+			except:
+				return None
 		self.execute(query, params, commit=commit)
 		return int(self.cursor().lastrowid)
 
@@ -333,19 +328,75 @@ class SQL:
 		self.execute(query, params, commit=True)
 		return int(self.cursor().rowcount)
 
+	def delete(self, table: str, filters: dict = lambda: {}) -> int:
+		''' Executes a delete operation and returns the number of affected rows, specifying
+		a filters list, i.e. `{'a': 4, 'b': None}` will be translated into `WHERE A = 4 AND B = NULL`. '''
+		query = 'DELETE FROM %s ' % table
+		params = []
+		if len(filters):
+			values = []
+			for k, v in filters.items():
+				values.append(k + '=%s ')
+				params.append(v)
+			query += 'WHERE ' + ' AND '.join(values)
+		self.execute(query, params, commit=True)
+		return int(self.cursor().rowcount)
+
 	def escape(self, value: Any, is_literal: bool = True) -> str:
 		''' Escapes the given value for its injection into the SQL query. By default,
 		the data `is_literal=True`, which will wrap strings with quotes for its insertion. '''
-		if value is None:
-			value = 'NULL'
-		else:
-			self.cursor()  # force initialization
-			value = self._connection.converter.escape(str(value))
+		if self.engine == 'postgre':
+			from psycopg import sql
 			if is_literal:
-				value = '"%s"' % value
+				value = sql.quote(value)
+			else:
+				self.cursor()  # force initialization
+				value = sql.Identifier(value).as_string(self._connection)
+		else:
+			if value is None:
+				value = 'NULL'
+			else:
+				self.cursor()  # force initialization
+				value = self._connection.converter.escape(str(value))
+				if is_literal:
+					value = '"%s"' % value
 		return value
 
-from re import compile, IGNORECASE
+def _mysql_converter():
+	''' Simpler MySQL converter that returns some bytes as strings and decimals as floats. '''
+	from mysql.connector.constants import FieldFlag
+	from mysql.connector.conversion import MySQLConverter as NativeConverter
+
+	class MySQLConverter(NativeConverter):
+		def row_to_python(self, row, fields):
+			res = super().row_to_python(row, fields)
+			return res
+
+		def _BLOB_to_python(self, value, dsc=None):
+			"""Convert BLOB data type to Python"""
+			return self._STRING_to_python(value, dsc)
+		_LONG_BLOB_to_python = _BLOB_to_python
+		_MEDIUM_BLOB_to_python = _BLOB_to_python
+		_TINY_BLOB_to_python = _BLOB_to_python
+
+		def _DECIMAL_to_python(self, value, desc=None):
+			return float(value.decode(self.charset))
+		_NEWDECIMAL_to_python = _DECIMAL_to_python
+
+		def _float64_to_mysql(self, value, desc=None):
+			return float(value)
+
+		def _STRING_to_python(self, value, dsc=None):
+			res = super(MySQLConverter, self)._STRING_to_python(value, dsc)
+			if dsc[7] & FieldFlag.BINARY:
+				return res.decode(self.charset)
+			return res
+		_VAR_STRING_to_python = _STRING_to_python
+
+		def _timestamp_to_mysql(self, value, desc=None):
+			return value.strftime('%Y-%m-%d %H:%M:%S').encode(self.charset)
+
+	return MySQLConverter
 
 class Excel:
 	''' Pandas Excel backend. '''
