@@ -3,7 +3,7 @@ def getch():
 	from os import name
 	if name == 'nt':  # Windows-based systems
 		from msvcrt import getch as ms_getch
-		res = ms_getch()
+		res = ms_getch().decode('utf-8')
 	else:  # POSIX-based systems
 		from tty import setcbreak
 		from sys import stdin
@@ -23,7 +23,7 @@ _cprint_palette = {
 	'light_green': 92, 'light_yellow': 93, 'light_blue': 94, 'light_magenta': 95,
 	'light_cyan': 96, 'white': 97
 }
-def cprint(*args: tuple, fg='default', bg='default', **kwargs):
+def cprint(*args, fg='default', bg='default', **kwargs):
 	''' Same syntax as print, with two optional parameters `fg` and `bg` to change
 	the print color. Available colors are: default, black, red, green, yellow, blue,
 	magenta, cyan, light_gray, dark_gray, light_red, light_green, light_yellow, light_blue,
@@ -31,6 +31,5 @@ def cprint(*args: tuple, fg='default', bg='default', **kwargs):
 	print('\033[%dm\033[%dm%s\033[0m' % (
 		_cprint_palette[fg],
 		_cprint_palette[bg] + 10,
-		' '.join(args)),
-		**kwargs
-	)
\ No newline at end of file
+		' '.join(map(str, args)),
+	), **kwargs)
\ No newline at end of file
